import 'web-streams-polyfill/polyfill/es5';
import { ProgressData } from './utility';
export declare enum BodyRequestMethods {
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    DELETE = "DELETE"
}
export interface RequestOptions {
    withCredentials?: boolean;
    timeout?: number;
    responseType?: XMLHttpRequestResponseType;
}
export interface Request extends RequestOptions {
    method?: 'HEAD' | 'GET' | keyof typeof BodyRequestMethods;
    path: string | URL;
    headers?: HeadersInit;
    body?: BodyInit | HTMLFormElement | any;
    signal?: AbortSignal;
}
export interface Response<B = Request['body']> {
    status: number;
    statusText: string;
    headers: Record<string, string | object>;
    body?: B;
}
export declare class HTTPError<B = Request['body']> extends URIError {
    request: Request;
    response: Response<B>;
    constructor(message: string, request: Request, response: Response<B>);
}
export type LinkHeader = Record<string, {
    URI: string;
    rel: string;
    title?: string;
}>;
export declare const headerParser: {
    Link: (value: string) => LinkHeader;
};
export declare const parseHeaders: (raw: string) => Response["headers"];
export declare function parseBody<T>(raw: string, contentType: string): T;
export interface RequestResult<B> {
    response: Promise<Response<B>>;
    upload?: AsyncGenerator<ProgressData>;
    download: AsyncGenerator<ProgressData>;
}
export declare function requestXHR<B>({ method, path, headers, body, signal, ...rest }: Request): RequestResult<B>;
export declare function requestFetch<B>({ path, method, headers, withCredentials, body, signal, timeout, responseType }: Request): RequestResult<B>;
export declare function parseResponse<B>(responsePromise: Promise<globalThis.Response>, streamPromise: Promise<ReadableStream<Uint8Array>>, responseType: Request['responseType']): Promise<Response<B>>;
export declare function parseFetchBody<B>(stream: ReadableStream<Uint8Array>, contentType: string, responseType: Request['responseType']): Promise<B>;
export declare function iterateFetchBody(responsePromise: Promise<globalThis.Response>, bodyPromise: Promise<ReadableStream<Uint8Array>>): AsyncGenerator<{
    total: number;
    loaded: any;
}, void, unknown>;
export declare const request: <B>(options: Request) => RequestResult<B>;
